"""equal operation"""


def equal(obj1, obj2):
    """Check if two JSON objects are equal.
    :param obj1 first JSON object
    :param obj2 second JSON object
    :return True if obj1 == obj2
    """
    if type(obj1) != type(obj2):
        return obj1 == obj2
    if isinstance(obj1, dict):
        if len(obj1) != len(obj2):
            return False
        for k in obj1:
            if k not in obj2 or not equal(obj1[k], obj2[k]):
                return False
        return True
    elif isinstance(obj1, list):
        if len(obj1) != len(obj2):
            return False
        marks = [0 for _ in range(len(obj2))]
        for s1 in obj1:
            found = False
            for i in range(len(obj2)):
                if marks[i] == 0 and equal(s1, obj2[i]):
                    found = True
                    marks[i] = 1
                    break
            if not found:
                return False
        return True
    else:
        return obj1 == obj2





"""set intersection"""

from .equal import equal


def intersection(obj1, obj2):
    """intersection operation for 2 JSON objects.
    :param obj1: first json object
    :param obj2: second json object
    :return: a json object that reflects "obj1 & ojb2"
    """
    if type(obj1) == type(obj2):
        if isinstance(obj1, dict):
            res = dict()
            for k in obj1:
                if k in obj2:
                    tmp = intersection(obj1[k], obj2[k])
                    if tmp:
                        res[k] = tmp
            if len(res) > 0:
                return res
        elif isinstance(obj1, list) or isinstance(obj1, tuple):
            res = list()
            if len(obj1) == 1 and len(obj2) == 1:
                ele = intersection(obj1[0], obj2[0])
                if ele and len(ele):
                    res.append(ele)
            else:
                mark = [0 for _ in range(len(obj2))]
                for v in obj1:
                    for i in range(len(mark)):
                        if mark[i] == 0 and equal(v, obj2[i]):
                            mark[i] = 1
                            res.append(v)
            if len(res) > 0:
                return res
        elif obj1 == obj2:
            return obj1
    return None




"""set difference"""

from .equal import equal


def subtract(obj1, obj2):
    if type(obj1) == type(obj2):
        if isinstance(obj1, dict):
            res = dict()
            for k in obj1:
                if k in obj2:
                    tmp = subtract(obj1[k], obj2[k])
                    if tmp:
                        res[k] = tmp
                else:
                    res[k] = obj1[k]
            if len(res) > 0:
                return res
        elif isinstance(obj1, list) or isinstance(obj1, tuple):
            res = list()
            if len(obj1) == 1 and len(obj2) == 1:
                ele = subtract(obj1[0], obj2[0])
                if ele and len(ele):
                    res.append(ele)
            else:
                for v in obj1:
                    found = False
                    for i in range(len(obj2)):
                        if equal(v, obj2[i]):
                            found = True
                            break
                    if not found:
                        res.append(v)
            if len(res) > 0:
                return res
        else:
            if obj1 == obj2:
                return None
            return obj1
    return None



import argparse
import json
import sys
from inspect import getmembers, isfunction
import json_operator


_operations = getmembers(json_operator, isfunction)
_op_keys = list(k for k,v in _operations)
_op_map = {k:v for k,v in _operations}


def _get_parser():
    parser = argparse.ArgumentParser(add_help=False,
        description='command utility to operate on JSON objects. Supported operators: %s.' % ", ".join(_op_keys))
    parser.add_argument('operator', nargs='?', action='store', help='action command')
    parser.add_argument('--lt', action='store', help='left-side JSON text. Overwrite --lf option')
    parser.add_argument('--rt', action='store', help='right-side JSON text. Overwrite --rf option')
    parser.add_argument('--lf', action='store', help='left-side file where JSON is stored')
    parser.add_argument('--rf', action='store', help='right-side file where JSON is stored')
    parser.add_argument("--out", action='store', help='output file. Default is stdout')
    parser.add_argument('--help', action='help', help='print this help message')
    return parser


def _get_json(f, text):
    if text:
        return json.loads(text)
    with open(f, "r") as fh:
        data = fh.read()
    return json.loads(data)


def main():
    parser = _get_parser()
    args = parser.parse_args(sys.argv[1:])
    if hasattr(args, "help") and args.help:
        parser.print_usage()
        return 0
    if args.operator not in _op_map:
        print("Invalid operator. Use --help to check help menu. Supported operators are: %s" % ", ".join(_op_keys))
        return 1
    if not args.lt and not args.lf:
        print("No left-side JSON, use --lt or --lf")
        return 1
    if not args.rt and not args.rf:
        print("No right-side JSON, use --rt or --rf")
        return 1
    left = _get_json(args.lf, args.lt)
    right = _get_json(args.rf, args.rt)
    res = _op_map[args.operator](left, right)
    data = json.dumps(res)
    if args.out:
        with open(args.out, "wt") as f:
            f.write(data)
    else:
        print(data)


if __name__ == "__main__":
    main()
